@@ -13,6 +13,12 @@ import logging
 from typing import Iterable
 
 
+def empty(*dims):
+    if len(dims)==1:
+        return [[] for x in range(dims[0])]
+    else:
+        return [empty(*dims[1:]) for _ in range(dims[0])]
+
 
 class AttrDict(dict):
     def __init__(self, *args, **kwargs):
@@ -59,7 +65,6 @@ class HDF5Decoder():
                 raise ValueError('can only unpack .mat')
         return d
     
-   
     def unpack_mat(self, hdf5, depth=0):
         """
         unpack a h5py entry: if it's a group expand,
@@ -71,14 +76,14 @@ class HDF5Decoder():
             raise RecursionError("Maximum number of 99 recursions reached.")
         if isinstance(hdf5, (h5py._hl.group.Group)):
             d = self._dict_class()
+
             for key in hdf5:
                 matlab_class = hdf5[key].attrs.get('MATLAB_class')
                 elem   = hdf5[key]
                 unpacked = self.unpack_mat(elem, depth=depth+1)
-                if matlab_class==b'struct' and len(elem)>1:
-
+                if matlab_class==b'struct' and len(elem)>1 and \
+                isinstance(unpacked, dict):
                     values = unpacked.values()
-
                     # we can only pack them together in MATLAB style if
                     # all subitems are the same lengths.
                     # MATLAB is a bit confusing here, and I hope this is
@@ -105,20 +110,18 @@ class HDF5Decoder():
 
             return d
         elif isinstance(hdf5, h5py._hl.dataset.Dataset):
-            return self.convert_mat(hdf5)
+            return self.convert_mat(hdf5, depth)
         else:
             raise Exception(f'Unknown hdf5 type: {key}:{type(hdf5)}')
 
-
+    # @profile
     def _has_refs(self, dataset):
-        if len(dataset)==0: return False
-        if not isinstance(dataset[0], np.ndarray): return False
+        if len(dataset.shape)<2: return False
         if isinstance(dataset[0][0], h5py.h5r.Reference):  
             return True
         return False
 
-
-    def convert_mat(self, dataset):
+    def convert_mat(self, dataset, depth):
         """
         Converts h5py.dataset into python native datatypes
         according to the matlab class annotation
@@ -135,7 +138,7 @@ class HDF5Decoder():
         if self._has_refs(dataset):
             mtype='cell'
         elif 'MATLAB_empty' in dataset.attrs.keys() and \
-            dataset.attrs['MATLAB_class'].decode()=='cell':
+            dataset.attrs['MATLAB_class'].decode()in ['cell', 'struct']:
             mtype = 'empty'
         else:
             mtype = dataset.attrs['MATLAB_class'].decode()
@@ -148,7 +151,7 @@ class HDF5Decoder():
                 # some weird style MATLAB have no refs, but direct floats or int
                 if isinstance(ref, Iterable):
                     for r in ref:
-                        entry = self.unpack_mat(self.refs.get(r))
+                        entry = self.unpack_mat(self.refs.get(r), depth+1)
                         row.append(entry)
                 else:
                     row = [ref]
@@ -160,7 +163,7 @@ class HDF5Decoder():
 
         elif mtype=='empty':
             dims = [x for x in dataset]
-            return np.empty(dims)
+            return empty(*dims)
 
         elif mtype=='char': 
             string_array = np.array(dataset).ravel()
@@ -204,7 +207,7 @@ class HDF5Decoder():
             return None
         
             
-def loadmat(filename, use_attrdict=True, only_load=None, verbose=True):
+def loadmat(filename, use_attrdict=False, only_load=None, verbose=True):
     """
     Loads a MATLAB 7.3 .mat file, which is actually a
     HDF5 file with some custom matlab annotations inside
@@ -238,5 +241,5 @@ if __name__=='__main__':
     # d = loadmat('../tests/testfile5.mat', use_attrdict=True)
 
 
-    file = 'C:/Users/Simon/Desktop/mat7.3/tests/testfile6.mat'
-    data = loadmat(file)
\ No newline at end of file
+    file = '../tests/testfile8.mat'
+    data = loadmat(file)
